# coding: utf-8
from __future__ import absolute_import
from __future__ import unicode_literals

__author__ = "Martin Larralde"
__author_email__ = "martin.larralde@ens-cachan.fr"
__version__ = "2.3.4"
__license___ = "GPLv3+"
